@@ -22,14 +22,18 @@ import newrelic.core.transaction_node
 import newrelic.core.database_node
 import newrelic.core.error_node
 
-import newrelic.api.time_trace
-
 from newrelic.core.stats_engine import CustomMetrics
 from newrelic.core.transaction_cache import transaction_cache
 from newrelic.core.thread_utilization import utilization_tracker
 
+from .time_trace import TimeTrace
+
 _logger = logging.getLogger(__name__)
 
+class Sentinel(TimeTrace):
+    def __init__(self):
+        super(Sentinel, self).__init__(None)
+
 class Transaction(object):
 
     STATE_PENDING = 0
@@ -142,6 +146,11 @@ class Transaction(object):
         self.referring_transaction_guid = None
         self.record_tt = False
 
+        self.synthetics_resource_id = None
+        self.synthetics_job_id = None
+        self.synthetics_monitor_id = None
+        self.synthetics_header = None
+
         self._custom_metrics = CustomMetrics()
 
         self._profile_samples = deque()
@@ -236,7 +245,7 @@ class Transaction(object):
         # dummy time trace object and when done we will
         # just grab what we need from that.
 
-        self._node_stack.append(newrelic.api.time_trace.TimeTrace(None))
+        self._node_stack.append(Sentinel())
 
         return self
 
@@ -247,6 +256,20 @@ class Transaction(object):
         if not self.enabled:
             return
 
+        # Ensure that we are actually back at the top of
+        # transaction call stack. Assume that it is an
+        # instrumentation error and return with hope that
+        # will recover later.
+
+        if not len(self._node_stack) == 1:
+            _logger.error('Runtime instrumentation error. Exiting the '
+                    'transaction but the node stack is not empty. '
+                    'Node stack is %r. Report this issue to New Relic '
+                    'support.\n%s', self._node_stack, ''.join(
+                    traceback.format_stack()[:-1]))
+
+            return
+
         # Mark as stopped and drop the transaction from
         # thread/coroutine local storage.
         #
@@ -455,6 +478,10 @@ class Transaction(object):
                 client_cross_process_id=self.client_cross_process_id,
                 referring_transaction_guid=self.referring_transaction_guid,
                 record_tt = self.record_tt,
+                synthetics_resource_id = self.synthetics_resource_id,
+                synthetics_job_id = self.synthetics_job_id,
+                synthetics_monitor_id = self.synthetics_monitor_id,
+                synthetics_header = self.synthetics_header,
                 )
 
         # Clear settings as we are all done and don't need it
@@ -829,8 +856,21 @@ class Transaction(object):
 
     def _pop_current(self, node):
         last = self._node_stack.pop()
-        assert last == node
+
+        try:
+            assert last == node
+
+        except Exception:
+            _logger.error('Runtime instrumentation error. Object on '
+                    'transaction node stack when removing last value is '
+                    'not the expected one, found %r, expected %r. Report '
+                    'this issue to New Relic support.\n%s', last, node,
+                    ''.join(traceback.format_stack()[:-1]))
+
+            raise
+
         parent = self._node_stack[-1]
+
         return parent
 
     def _process_node(self, node):
@@ -872,8 +912,8 @@ class Transaction(object):
         self._cpu_user_time_end = os.times()[0]
 
     def add_custom_parameter(self, name, value):
-        if not self._settings: 
-            return 
+        if not self._settings:
+            return
 
         if not self._settings.high_security:
             self._custom_params[name] = value
@@ -932,13 +972,13 @@ class Transaction(object):
 
         print >> file, 'Node Stack:'
         for node in self._node_stack[1:]:
-            node.dump(file)
+            print >> file, node
 
-
-def current_transaction():
+def current_transaction(active_only=True):
     current = transaction_cache().current_transaction()
-    if current and (current.ignore_transaction or current.stopped):
-        return None
+    if active_only:
+        if current and (current.ignore_transaction or current.stopped):
+            return None
     return current
 
 def transaction():
