@@ -9,6 +9,8 @@ import sys
 import inspect
 import functools
 
+from ..packages import six
+
 from ..packages.wrapt import (ObjectProxy as _ObjectProxy,
         FunctionWrapper as _FunctionWrapper,
         BoundFunctionWrapper as _BoundFunctionWrapper)
@@ -151,7 +153,7 @@ wrap_callable = FunctionWrapper
 # Helper functions for performing monkey patching.
 
 def resolve_path(module, name):
-    if not inspect.ismodule(module):
+    if isinstance(module, six.string_types):
         __import__(module)
         module = sys.modules[module]
 
@@ -163,7 +165,30 @@ def resolve_path(module, name):
     original = getattr(parent, attribute)
     for attribute in path[1:]:
         parent = original
-        original = getattr(original, attribute)
+
+        # We can't just always use getattr() because in doing
+        # that on a class it will cause binding to occur which
+        # will complicate things later and cause some things not
+        # to work. For the case of a class we therefore access
+        # the __dict__ directly. To cope though with the wrong
+        # class being given to us, or a method being moved into
+        # a base class, we need to walk the class heirarchy to
+        # work out exactly which __dict__ the method was defined
+        # in, as accessing it from __dict__ will fail if it was
+        # not actually on the class given. Fallback to using
+        # getattr() if we can't find it. If it truly doesn't
+        # exist, then that will fail.
+
+        if inspect.isclass(original):
+            for cls in inspect.getmro(original):
+                if attribute in vars(original):
+                    original = vars(original)[attribute]
+                    break
+            else:
+                original = getattr(original, attribute)
+
+        else:
+            original = getattr(original, attribute)
 
     return (parent, attribute, original)
 
