@@ -1,6 +1,7 @@
 import sys
 import threading
 import logging
+import functools
 
 from newrelic.packages import six
 
@@ -16,7 +17,7 @@ _logger = logging.getLogger(__name__)
 
 _boolean_states = {
    '1': True, 'yes': True, 'true': True, 'on': True,
-   '0': False, 'no': False, 'false': False, 'off': False 
+   '0': False, 'no': False, 'false': False, 'off': False
 }
 
 def _setting_boolean(value):
@@ -112,7 +113,7 @@ def browser_timing_middleware(request, response):
 
     cdisposition = response.get('Content-Disposition', '').lower()
 
-    if cdisposition.split(';')[0].strip().lower() == 'attachment': 
+    if cdisposition.split(';')[0].strip().lower() == 'attachment':
         return response
 
     # No point continuing if header is empty. This can occur if
@@ -735,7 +736,8 @@ def instrument_django_core_servers_basehttp(module):
     # not going to be changed, could just patch it to fix
     # problem and the instrumentation we need.
 
-    if hasattr(module.ServerHandler, 'run'):
+    if (not hasattr(module, 'simple_server') and
+            hasattr(module.ServerHandler, 'run')):
 
         # Patch the server to make it work properly.
 
@@ -923,9 +925,139 @@ def _nr_wrapper_django_template_base_Library_inclusion_tag_(wrapped,
     return _nr_wrapper_django_inclusion_tag_decorator_(
             wrapped(*args, **kwargs))
 
+@function_wrapper
+def _nr_wrapper_django_template_base_InclusionNode_render_(wrapped,
+        instance, args, kwargs):
+
+    transaction = current_transaction()
+
+    if transaction is None:
+        return wrapped(*args, **kwargs)
+
+    if wrapped.__self__ is None:
+        return wrapped(*args, **kwargs)
+
+    file_name = getattr(wrapped.__self__, '_nr_file_name', None)
+
+    if file_name is None:
+        return wrapped(*args, **kwargs)
+
+    name = wrapped.__self__._nr_file_name
+
+    with FunctionTrace(transaction, name, 'Template/Include'):
+        return wrapped(*args, **kwargs)
+
+def _nr_wrapper_django_template_base_generic_tag_compiler_(wrapped, instance,
+        args, kwargs):
+
+    if wrapped.__code__.co_argcount > 6:
+        # Django > 1.3.
+
+        def _bind_params(parser, token, params, varargs, varkw, defaults,
+                name, takes_context, node_class, *args, **kwargs):
+            return node_class
+    else:
+        # Django <= 1.3.
+
+        def _bind_params(params, defaults, name, node_class, parser, token,
+                *args, **kwargs):
+            return node_class
+
+    node_class = _bind_params(*args, **kwargs)
+
+    if node_class.__name__ == 'InclusionNode':
+        result = wrapped(*args, **kwargs)
+
+        result.render = (
+                _nr_wrapper_django_template_base_InclusionNode_render_(
+                result.render))
+
+        return result
+
+    return wrapped(*args, **kwargs)
+
+def _nr_wrapper_django_template_base_Library_tag_(wrapped, instance,
+        args, kwargs):
+
+    def _bind_params(name=None, compile_function=None, *args, **kwargs):
+        return compile_function
+
+    compile_function = _bind_params(*args, **kwargs)
+
+    if not callable(compile_function):
+        return wrapped(*args, **kwargs)
+
+    def _get_node_class(compile_function):
+
+        node_class = None
+
+        # Django >= 1.4 uses functools.partial
+
+        if isinstance(compile_function, functools.partial):
+            node_class = compile_function.keywords.get('node_class')
+
+        # Django < 1.4 uses their home-grown "curry" function,
+        # not functools.partial.
+
+        if (hasattr(compile_function, 'func_closure')
+                and hasattr(compile_function, '__name__')
+                and compile_function.__name__ == '_curried'):
+
+            # compile_function here is generic_tag_compiler(), which has been
+            # curried. To get node_class, we first get the function obj, args,
+            # and kwargs of the curried function from the cells in
+            # compile_function.func_closure. But, the order of the cells
+            # is not consistent from platform to platform, so we need to map
+            # them to the variables in compile_function.__code__.co_freevars.
+
+            cells = dict(zip(compile_function.__code__.co_freevars,
+                    (c.cell_contents for c in compile_function.func_closure)))
+
+            # node_class is the 4th arg passed to generic_tag_compiler()
+
+            if 'args' in cells and len(cells['args']) > 3:
+                node_class = cells['args'][3]
+
+        return node_class
+
+    node_class = _get_node_class(compile_function)
+
+    if node_class is None or node_class.__name__ != 'InclusionNode':
+        return wrapped(*args, **kwargs)
+
+    # Climb stack to find the file_name of the include template.
+    # While you only have to go up 1 frame when using python with
+    # extensions, pure python requires going up 2 frames.
+
+    file_name = None
+    stack_levels = 2
+
+    for i in range(1, stack_levels + 1):
+        frame = sys._getframe(i)
+
+        if ('generic_tag_compiler' in frame.f_code.co_names
+                and 'file_name' in frame.f_code.co_freevars):
+            file_name = frame.f_locals.get('file_name')
+
+    if file_name is None:
+        return wrapped(*args, **kwargs)
+
+    if isinstance(file_name, module_django_template_base.Template):
+        file_name = file_name.name
+
+    node_class._nr_file_name = file_name
+
+    return wrapped(*args, **kwargs)
+
 def instrument_django_template_base(module):
     global module_django_template_base
     module_django_template_base = module
 
+    wrap_function_wrapper(module, 'generic_tag_compiler',
+            _nr_wrapper_django_template_base_generic_tag_compiler_)
+
+    wrap_function_wrapper(module, 'Library.tag',
+            _nr_wrapper_django_template_base_Library_tag_)
+
     wrap_function_wrapper(module, 'Library.inclusion_tag',
             _nr_wrapper_django_template_base_Library_inclusion_tag_)
